@@ -4,7 +4,7 @@ import pytest
 import numpy as np
 import pandas as pd
 
-from formulae.design_matrices import design_matrices
+from formulae.matrices import design_matrices
 
 # TODO: See interaction names.. they don't always work as expected
 @pytest.fixture(scope="module")
@@ -24,6 +24,26 @@ def data():
     return data
 
 
+@pytest.fixture(scope="module")
+def pixel():
+    """
+    X-ray pixel intensities over time dataset from R nlme package.
+    The output is a subset of this dataframe.
+    """
+    from os.path import dirname, join
+
+    data_dir = join(dirname(__file__), "data")
+    data = pd.read_csv(join(data_dir, "Pixel.csv"))
+
+    data["Dog"] = data["Dog"].astype("category")
+    data["day"] = data["day"].astype("category")
+    data = data[data["Dog"].isin([1, 2, 3])]
+    data = data[data["day"].isin([2, 4, 6])]
+    data = data.sort_values(["Dog", "Side", "day"])
+    data = data.reset_index(drop=True)
+    return data
+
+
 def compare_dicts(d1, d2):
     if len(d1) != len(d2):
         return False
@@ -382,12 +402,6 @@ def test_built_in_transforms(data):
         "C(x3)[4]",
     ]
 
-    # Specify reference -> it is converted into 0-1 variable
-    dm = design_matrices("y ~ C(x3, 3)", data)
-    assert dm.common.terms_info["C(x3, 3)"]["type"] == "numeric"
-    assert dm.common.terms_info["C(x3, 3)"]["full_names"] == ["C(x3, 3)"]
-    assert all(dm.common["C(x3, 3)"][:, 0] == np.where(data["x3"] == 3, 1, 0))
-
     # Specify levels, different to observed
     lvls = [3, 2, 4, 1]
     dm = design_matrices("y ~ C(x3, levels=lvls)", data)
@@ -464,3 +478,230 @@ def test_categoric_group_specific():
         "C(age_grp)[1]|BMI",
         "C(age_grp)[2]|BMI",
     ]
+
+
+def test_interactions_in_group_specific(pixel):
+    # We have group specific terms with the following characteristics
+    # 1. expr=categoric, factor=categoric
+    # 2. expr=intercept, factor=categoric
+    # 3. expr=intercept, factor=interaction between categorics
+    # The desing matrices used for the comparison are loaded from text files.
+    # The encoding is implicitly checked when comparing names.
+
+    from os.path import dirname, join
+
+    data_dir = join(dirname(__file__), "data/group_specific")
+    slope_by_dog_original = np.loadtxt(join(data_dir, "slope_by_dog.txt"))
+    intercept_by_side_original = np.loadtxt(join(data_dir, "intercept_by_side.txt"))
+    intercept_by_side_dog_original = np.loadtxt(join(data_dir, "intercept_by_side_dog.txt"))
+    dog_and_side_by_day_original = np.loadtxt(join(data_dir, "dog_and_side_by_day.txt"))
+
+    dm = design_matrices("pixel ~ day +  (0 + day | Dog) + (1 | Side/Dog)", pixel)
+    slope_by_dog = dm.group["day|Dog"]
+    intercept_by_side = dm.group["1|Side"]
+    intercept_by_side_dog = dm.group["1|Side:Dog"]
+
+    # Assert values in the design matrix
+    assert (slope_by_dog == slope_by_dog_original).all()
+    assert (intercept_by_side == intercept_by_side_original).all()
+    assert (intercept_by_side_dog == intercept_by_side_dog_original).all()
+
+    # Assert full names
+    names = [f"day[{d}]|{g}" for g in [1, 2, 3] for d in [2, 4, 6]]
+    assert dm.group.terms_info["day|Dog"]["full_names"] == names
+    names = [f"1|Side[{s}]" for s in ["L", "R"]]
+    assert dm.group.terms_info["1|Side"]["full_names"] == names
+    names = [f"1|Side:Dog[{s}:{d}]" for s in ["L", "R"] for d in [1, 2, 3]]
+    assert dm.group.terms_info["1|Side:Dog"]["full_names"] == names
+
+    # Another design matrix
+    dm = design_matrices("(0 + Dog:Side | day)", pixel)
+    dog_and_side_by_day = dm.group["Dog:Side|day"]
+
+    # Assert values in the design matrix
+    assert (dog_and_side_by_day == dog_and_side_by_day_original).all()
+
+    # Assert full names
+    names = [f"Dog[{d}]:Side[{s}]|{g}" for g in [2, 4, 6] for d in [1, 2, 3] for s in ["L", "R"]]
+    assert dm.group.terms_info["Dog:Side|day"]["full_names"] == names
+
+
+def test_prop_response():
+    data = pd.DataFrame(
+        {
+            "x": np.array([1.6907, 1.7242, 1.7552, 1.7842, 1.8113, 1.8369, 1.8610, 1.8839]),
+            "n": np.array([59, 60, 62, 56, 63, 59, 62, 60]),
+            "y": np.array([6, 13, 18, 28, 52, 53, 61, 60]),
+        }
+    )
+
+    response = design_matrices("prop(y, n) ~ x", data).response
+
+    assert response.type == "prop"
+    assert response.design_vector.shape == (8, 2)
+    assert (np.less_equal(response.design_vector[:, 0], response.design_vector[:, 1])).all()
+
+
+def test_prop_response_fails():
+    # x larger than n
+    with pytest.raises(ValueError):
+        design_matrices("prop(x, n) ~ 1", pd.DataFrame({"x": [2, 3], "n": [1, 2]}))
+
+    # x and/or n not integer
+    with pytest.raises(ValueError):
+        design_matrices("prop(x, n) ~ 1", pd.DataFrame({"x": [2, 3.3], "n": [4, 4]}))
+
+    with pytest.raises(ValueError):
+        design_matrices("prop(x, n) ~ 1", pd.DataFrame({"x": [2, 3], "n": [4.3, 4]}))
+
+
+def test_categoric_responses():
+    data = pd.DataFrame(
+        {
+            "y1": np.random.choice(["A", "B", "C"], size=30),
+            "y2": np.random.choice(["A", "B"], size=30),
+            "x": np.random.normal(size=30),
+        }
+    )
+
+    # Multi-level response
+    response = design_matrices("y1 ~ x", data).response
+    assert list(np.unique(response.design_vector)) == [0, 1, 2]
+    assert response.levels == ["A", "B", "C"]
+    assert response.binary is False
+    assert response.baseline == "A"
+    assert response.success is None
+
+    # Multi-level response, explicitly converted to binary
+    response = design_matrices("y1['A'] ~ x", data).response
+    assert list(np.unique(response.design_vector)) == [0, 1]
+    assert response.levels == ["A", "B", "C"]
+    assert response.binary is True
+    assert response.baseline is None
+    assert response.success == "A"
+
+    # Default binary response
+    response = design_matrices("y2 ~ x", data).response
+    assert list(np.unique(response.design_vector)) == [0, 1]
+    assert response.levels == ["A", "B"]
+    assert response.binary is True
+    assert response.baseline is None
+    assert response.success == "A"
+
+    # Binary response with explicit level
+    response = design_matrices("y2['B'] ~ x", data).response
+    assert list(np.unique(response.design_vector)) == [0, 1]
+    assert response.levels == ["A", "B"]
+    assert response.binary is True
+    assert response.baseline is None
+    assert response.success == "B"
+
+
+def test_binary_function():
+    size = 100
+    data = pd.DataFrame(
+        {
+            "y": np.random.randint(0, 5, size=size),
+            "x": np.random.randint(5, 10, size=size),
+            "g": np.random.choice(["a", "b", "c"], size=size),
+        }
+    )
+
+    # String value
+    term = design_matrices("y ~ binary(g, 'c')", data).common["binary(g, c)"].squeeze()
+    assert np.array_equal(np.where(term == 1), np.where(data["g"] == "c"))
+
+    # Numeric value
+    term = design_matrices("y ~ binary(x, 7)", data).common["binary(x, 7)"].squeeze()
+    assert np.array_equal(np.where(term == 1), np.where(data["x"] == 7))
+
+    # Variable name
+    # string
+    m = "b"
+    term = design_matrices("y ~ binary(g, m)", data).common["binary(g, m)"].squeeze()
+    assert np.array_equal(np.where(term == 1), np.where(data["g"] == m))
+
+    # numeric
+    z = 8
+    term = design_matrices("y ~ binary(x, z)", data).common["binary(x, z)"].squeeze()
+    assert np.array_equal(np.where(term == 1), np.where(data["x"] == z))
+
+    # Pass nothing
+    term = design_matrices("y ~ binary(x)", data).common["binary(x)"].squeeze()
+    assert np.array_equal(np.where(term == 1), np.where(data["x"] == 5))
+
+    # Values not found in the variable
+    with pytest.raises(ValueError):
+        design_matrices("y ~ binary(g, 'Not found')", data)
+
+    with pytest.raises(ValueError):
+        design_matrices("y ~ binary(x, 999)", data)
+
+
+def test_C_function():
+    size = 100
+    data = pd.DataFrame(
+        {
+            "y": np.random.randint(0, 5, size=size),
+            "x": np.random.randint(5, 10, size=size),
+            "g": np.random.choice(["a", "b", "c"], size=size),
+        }
+    )
+
+    term = design_matrices("y ~ C(x)", data).common.terms_info["C(x)"]
+    assert term["type"] == "categoric"
+    assert term["levels"] == [5, 6, 7, 8, 9]
+    assert term["reference"] == 5
+
+    term = design_matrices("y ~ C(x, 7)", data).common.terms_info["C(x, 7)"]
+    assert term["type"] == "categoric"
+    assert term["levels"] == [7, 5, 6, 8, 9]
+    assert term["reference"] == 7
+
+    l = [6, 8, 5, 7, 9]
+    term = design_matrices("y ~ C(x, levels=l)", data).common.terms_info["C(x, levels = l)"]
+    assert term["type"] == "categoric"
+    assert term["levels"] == l
+    assert term["reference"] == 6
+
+    term = design_matrices("y ~ C(g)", data).common.terms_info["C(g)"]
+    assert term["type"] == "categoric"
+    assert term["levels"] == ["a", "b", "c"]
+    assert term["reference"] == "a"
+
+    term = design_matrices("y ~ C(g, 'c')", data).common.terms_info["C(g, c)"]
+    assert term["type"] == "categoric"
+    assert term["levels"] == ["c", "a", "b"]
+    assert term["reference"] == "c"
+
+    l = ["b", "c", "a"]
+    term = design_matrices("y ~ C(g, levels=l)", data).common.terms_info["C(g, levels = l)"]
+    assert term["type"] == "categoric"
+    assert term["levels"] == l
+    assert term["reference"] == "b"
+
+    with pytest.raises(ValueError):
+        design_matrices("y ~ C(g, 'c', levels=l)", data)
+
+
+def test_offset():
+    size = 100
+    data = pd.DataFrame(
+        {
+            "y": np.random.randint(0, 5, size=size),
+            "x": np.random.randint(5, 10, size=size),
+            "g": np.random.choice(["a", "b", "c"], size=size),
+        }
+    )
+
+    dm = design_matrices("y ~ offset(x)", data)
+    term = dm.common.terms_info["offset(x)"]
+    assert term["type"] == "offset"
+    assert term["full_names"] == ["offset(x)"]
+    assert (dm.common["offset(x)"].flatten() == data["x"]).all()
+
+    with pytest.raises(ValueError):
+        design_matrices("y ~ offset(g)", data)
+
+    with pytest.raises(ValueError):
+        design_matrices("offset(y) ~ x", data)
