@@ -11,7 +11,7 @@ def listify(obj):
     if obj is None:
         return []
     else:
-        return obj if isinstance(obj, (list, tuple, type(None))) else [obj]
+        return obj if isinstance(obj, (list, tuple)) else [obj]
 
 
 def flatten_list(nested_list):
